<?php

namespace Drupal\fixed_block_content\Entity;

use Drupal\Core\Config\Entity\ConfigEntityBase;
use Drupal\fixed_block_content\FixedBlockContentInterface;
use Drupal\block_content\BlockContentInterface;
use Drupal\block_content\Entity\BlockContent;

/**
 * Configuration entity for the fixed block content.
 *
 * @ConfigEntityType(
 *   id = "fixed_block_content",
 *   label = @Translation("Fixed block content"),
 *   config_prefix = "fixed_block_content",
 *   entity_keys = {
 *     "id" = "id",
 *     "label" = "title"
 *   },
 *   handlers = {
 *     "access" = "Drupal\fixed_block_content\FixedBlockContentAccessControlHandler",
 *     "list_builder" = "Drupal\fixed_block_content\FixedBlockContentListBuilder",
 *     "form" = {
 *       "add" = "Drupal\fixed_block_content\Form\FixedBlockContentForm",
 *       "edit" = "Drupal\fixed_block_content\Form\FixedBlockContentForm",
 *       "delete" = "Drupal\Core\Entity\EntityDeleteForm",
 *       "export" = "Drupal\fixed_block_content\Form\ExportConfirmForm",
 *       "import" = "Drupal\fixed_block_content\Form\ImportConfirmForm"
 *     }
 *   },
 *   links = {
 *     "collection" = "/admin/structure/block/block-content/fixed-block-content",
 *     "canonical" = "/admin/structure/block/block-content/fixed-block-content/manage/{fixed_block_content}",
 *     "edit-form" = "/admin/structure/block/block-content/fixed-block-content/manage/{fixed_block_content}/edit",
 *     "delete-form" = "/admin/structure/block/block-content/fixed-block-content/manage/{fixed_block_content}/delete",
 *     "export-form" = "/admin/structure/block/block-content/fixed-block-content/manage/{fixed_block_content}/export",
 *     "import-form" = "/admin/structure/block/block-content/fixed-block-content/manage/{fixed_block_content}/import"
 *   },
 *   config_export = {
 *     "id",
 *     "title",
 *     "block_content_bundle",
 *     "default_content"
 *   }
 * )
 */
class FixedBlockContent extends ConfigEntityBase implements FixedBlockContentInterface {

  /**
   * The block ID.
   *
   * @var string
   */
  protected $id;

  /**
   * The block title.
   *
   * @var string
   */
  protected $title;

  /**
   * The block content bundle.
   *
   * @var string
   */
  protected $block_content_bundle;

  /**
   * The serialized default content for this fixed block.
   *
   * @var string
   */
  protected $default_content;

  /**
   * The current block content linked to this fixed block.
   *
   * @var \Drupal\block_content\BlockContentInterface
   */
  protected $blockContent;

  /**
   * {@inheritdoc}
   */
  public function getBlockContent($create = TRUE) {
    if ($this->blockContent) {
      return $this->blockContent;
    }

    $block_content_storage = $this->entityTypeManager()->getStorage('block_content');
    /** @var \Drupal\Core\Database\StatementInterface $bids */
    $bids = \Drupal::database()->select('fixed_block_content', 'fbc')
      ->fields('fbc', ['bid'])
      ->range(0, 1)
      ->condition('fbc.fbid', $this->id)
      ->execute();

    // Block content not found. Create one.
    if ($bid = $bids->fetchField()) {
      $this->blockContent = $block_content_storage->load($bid);
    }

    if (!$this->blockContent && $create) {
      if (!empty($this->default_content)) {
        $this->exportDefaultContent();
      }
      else {
        /** @var \Drupal\block_content\BlockContentInterface $block_content */
        $new_block_content = $block_content_storage->create([
          'type' => $this->block_content_bundle,
          'info' => $this->title,
          'langcode' => $this->languageManager()->getDefaultLanguage()->getId(),
        ]);
        $new_block_content->enforceIsNew(TRUE);
        $new_block_content->setNewRevision(FALSE);
        $new_block_content->save();

        $this->setBlockContent($new_block_content);
      }
    }

    return $this->blockContent;
  }

  /**
   * Links a block content with this fixed block.
   *
   * Old block content, if exists, is not deleted.
   *
   * @param BlockContentInterface $block_content
   *   The new block content.
   */
  protected function setBlockContent(BlockContentInterface $block_content) {
    $this->blockContent = $block_content;
    \Drupal::database()->upsert('fixed_block_content')
      ->key('fbid')
      ->fields([
        'fbid' => $this->id,
        'bid' => $block_content->id(),
      ])
      ->execute();
  }

  /**
   * {@inheritdoc}
   */
  public function getBlockContentBundle() {
    return $this->block_content_bundle;
  }

  /**
   * {@inheritdoc}
   */
  public function exportDefaultContent() {
    // Does nothing if no default content present.
    if (empty($this->default_content)) {
      return;
    }

    // Delete the current block content.
    if ($curent_block = $this->getBlockContent(FALSE)) {
      $curent_block->delete();
    }

    // Create the new custom block from default contents.
    $new_block_content = \Drupal::service('serializer')->deserialize($this->default_content, 'Drupal\block_content\Entity\BlockContent', 'json', ['fixed_block_content' => $this]);
    $new_block_content->save();
    $this->setBlockContent($new_block_content);
  }

  /**
   * {@inheritdoc}
   */
  public function importDefaultContent() {
    $this->default_content = \Drupal::service('serializer')->serialize($this->getBlockContent(), 'json', ['fixed_block_content' => $this]);
    $this->save();
  }

  /**
   * {@inheritdoc}
   */
  public function calculateDependencies() {
    parent::calculateDependencies();
    $block_content = $this->getBlockContent();
    $this->addDependency($block_content->getConfigDependencyKey(), $block_content->getConfigDependencyName());
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function delete() {
    // Remove any fixed block links.
    \Drupal::database()->delete('fixed_block_content')
      ->condition('fbid', $this->id)
      ->execute();

    parent::delete();
  }
}
